function [r2xy, pvalxy, r2yx, pvalyx]=minifit(x, y, option, xx, yy)
%minifit(x, y)
% option 1 linear
% option 2 rbf
% option 3 piecewise linear (identity)

gamma=0.001;

if nargin<3
    option=1;
end
if nargin<4, xx=[]; yy=[]; end

x=(x-mean(x))/std(x);
y=(y-mean(y))/std(y);

if option<0
    h=figure;
    [h, A]=scplot(x, y, h);
    if option<-1, title(A(1,1), '');title(A(2,2), ''); end
    return
end


% Shows examples fo fit for didactic purposes
minx=min(x);
miny=min(y);
maxx=max(x);
maxy=max(y);

% Linear fit first direction
if option==1
    w=[x,ones(size(x))]\y;
    yhat=[x,ones(size(x))]*w;
elseif option==2 % RBF
    %yhat=fit_gp(x, y);
    [d, m]=train(kridge(['gamma=' num2str(gamma)]), data(x, y));
    %[d, m]=train(chain({standardize, kridge('degree=2')}), data(x, y));
    yhat=d.X;
else
    yhat=y;
end

r2xy=mean((y-yhat).^2)/var(y);

[pvalxy, resicorxy]=fasthsic((y-yhat), x);

figure; hold on
subplot(2,2,1)
plot(x, y, 'o', 'MarkerEdgeColor', 'b', 'MarkerFaceColor', 'b', 'MarkerSize', 5);
xlabel('A', 'FontSize', 16, 'FontWeight', 'bold');
ylabel('B', 'FontSize', 16, 'FontWeight', 'bold');
[~, i]=sort(x);
hold on
plot(x(i), yhat(i), 'r-', 'LineWidth', 2);
title(sprintf('R^2[B=F(A)]=%5.2f', r2xy), 'FontSize', 16, 'FontWeight', 'bold');
axis square
xlim([minx maxx]);
ylim([miny maxy]);

subplot(2,2,3)
plot(x, y-yhat, 'o', 'MarkerEdgeColor', 'b', 'MarkerFaceColor', 'b', 'MarkerSize', 5);
hold on
plot([minx, maxx], [0 0], 'r-', 'LineWidth', 2);
xlabel('A', 'FontSize', 16, 'FontWeight', 'bold');
ylabel('B-f(A)', 'FontSize', 16, 'FontWeight', 'bold');
title(sprintf('indep[B-F(A), A]=%5.2g', pvalxy), 'FontSize', 16, 'FontWeight', 'bold');
axis square
xlim([minx maxx]);

% reverse fit
if option ==1
    w=[y,ones(size(y))]\x;
    xhat=[y,ones(size(y))]*w;
elseif option==2 % RBF
    %xhat=fit_gp(y, x);
    [d, m]=train(kridge(['gamma=' num2str(gamma)]), data(y, x));
    xhat=d.X;
else
    xhat=x;
end

r2yx=mean((x-xhat).^2)/var(x);

[pvalyx, resicoryx]=fasthsic((x-xhat), y);

subplot(2,2,2)
plot(y, x, 'o', 'MarkerEdgeColor', 'b', 'MarkerFaceColor', 'b', 'MarkerSize', 5);
xlabel('B', 'FontSize', 16, 'FontWeight', 'bold');
ylabel('A', 'FontSize', 16, 'FontWeight', 'bold');
[~, i]=sort(y);
hold on
plot(y(i), xhat(i), 'g-', 'LineWidth', 2);
title(sprintf('R^2[A=F(B)]=%5.2f', r2yx), 'FontSize', 16, 'FontWeight', 'bold');
axis square
xlim([miny maxy]);
ylim([minx maxx]);

subplot(2,2,4)
plot(y, x-xhat, 'o', 'MarkerEdgeColor', 'b', 'MarkerFaceColor', 'b', 'MarkerSize', 5);
hold on
plot([miny, maxy], [0 0], 'g-', 'LineWidth', 2);
xlabel('B', 'FontSize', 16, 'FontWeight', 'bold');
ylabel('A-f(B)', 'FontSize', 16, 'FontWeight', 'bold');
title(sprintf('indep[A-F(B), B]=%5.2g', pvalyx), 'FontSize', 16, 'FontWeight', 'bold');
axis square
xlim([miny maxy]);



